"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getErrorMessage = exports.IndexedDB = exports.DBClosedError = void 0;
// From https://github.com/microsoft/vscode/blob/main/src/vs/base/browser/indexedDB.ts
class MissingStoresError extends Error {
    constructor(db) {
        super("Missing stores");
        this.db = db;
    }
}
class DBClosedError extends Error {
    constructor(dbName) {
        super(`IndexedDB database '${dbName}' is closed.`);
        this.code = "DBClosed";
    }
}
exports.DBClosedError = DBClosedError;
class IndexedDB {
    static create(name, version, stores) {
        return __awaiter(this, void 0, void 0, function* () {
            const database = yield IndexedDB.openDatabase(name, version, stores);
            return new IndexedDB(database, name);
        });
    }
    static openDatabase(name, version, stores) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield IndexedDB.doOpenDatabase(name, version, stores);
            }
            catch (err) {
                if (err instanceof MissingStoresError) {
                    console.info(`Attempting to recreate the IndexedDB once.`, name);
                    try {
                        // Try to delete the db
                        yield IndexedDB.deleteDatabase(err.db);
                    }
                    catch (error) {
                        console.error(`Error while deleting the IndexedDB`, getErrorMessage(error));
                        throw error;
                    }
                    return yield IndexedDB.doOpenDatabase(name, version, stores);
                }
                throw err;
            }
        });
    }
    static doOpenDatabase(name, version, stores) {
        return new Promise((c, e) => {
            const request = self.indexedDB.open(name, version);
            request.onerror = () => e(request.error);
            request.onsuccess = () => {
                const db = request.result;
                for (const store of stores) {
                    if (!db.objectStoreNames.contains(store)) {
                        console.error(`Error while opening IndexedDB. Could not find '${store}'' object store`);
                        e(new MissingStoresError(db));
                        return;
                    }
                }
                c(db);
            };
            request.onupgradeneeded = () => {
                const db = request.result;
                for (const store of stores) {
                    if (!db.objectStoreNames.contains(store)) {
                        db.createObjectStore(store);
                    }
                }
            };
        });
    }
    static deleteDatabase(indexedDB) {
        return new Promise((c, e) => {
            // Close any opened connections
            indexedDB.close();
            // Delete the db
            const deleteRequest = self.indexedDB.deleteDatabase(indexedDB.name);
            deleteRequest.onerror = () => e(deleteRequest.error);
            deleteRequest.onsuccess = () => c();
        });
    }
    constructor(database, name) {
        this.name = name;
        this.database = null;
        this.pendingTransactions = [];
        this.database = database;
    }
    hasPendingTransactions() {
        return this.pendingTransactions.length > 0;
    }
    close() {
        var _a;
        if (this.pendingTransactions.length) {
            this.pendingTransactions
                .splice(0, this.pendingTransactions.length)
                .forEach((transaction) => transaction.abort());
        }
        (_a = this.database) === null || _a === void 0 ? void 0 : _a.close();
        this.database = null;
    }
    runInTransaction(store, transactionMode, dbRequestFn) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.database) {
                throw new DBClosedError(this.name);
            }
            const transaction = this.database.transaction(store, transactionMode);
            this.pendingTransactions.push(transaction);
            return new Promise((c, e) => {
                transaction.oncomplete = () => {
                    if (Array.isArray(request)) {
                        c(request.map((r) => r.result));
                    }
                    else {
                        c(request.result);
                    }
                };
                transaction.onerror = () => e(transaction.error);
                transaction.onabort = () => e(transaction.error);
                const request = dbRequestFn(transaction.objectStore(store));
            }).finally(() => this.pendingTransactions.splice(this.pendingTransactions.indexOf(transaction), 1));
        });
    }
    getKeyValues(store, isValid) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.database) {
                throw new DBClosedError(this.name);
            }
            const transaction = this.database.transaction(store, "readonly");
            this.pendingTransactions.push(transaction);
            return new Promise((resolve) => {
                const items = new Map();
                const objectStore = transaction.objectStore(store);
                // Open a IndexedDB Cursor to iterate over key/values
                const cursor = objectStore.openCursor();
                if (!cursor) {
                    return resolve(items); // this means the `ItemTable` was empty
                }
                // Iterate over rows of `ItemTable` until the end
                cursor.onsuccess = () => {
                    if (cursor.result) {
                        // Keep cursor key/value in our map
                        if (isValid(cursor.result.value)) {
                            items.set(cursor.result.key.toString(), cursor.result.value);
                        }
                        // Advance cursor to next row
                        cursor.result.continue();
                    }
                    else {
                        resolve(items); // reached end of table
                    }
                };
                // Error handlers
                const onError = (error) => {
                    var _a;
                    console.error(`IndexedDB getKeyValues(): ${(_a = error === null || error === void 0 ? void 0 : error.message) !== null && _a !== void 0 ? _a : ""}`);
                    resolve(items);
                };
                cursor.onerror = () => onError(cursor.error);
                transaction.onerror = () => onError(transaction.error);
            }).finally(() => this.pendingTransactions.splice(this.pendingTransactions.indexOf(transaction), 1));
        });
    }
}
exports.IndexedDB = IndexedDB;
function getErrorMessage(err) {
    if (!err) {
        return "Error";
    }
    if (typeof err === "object" &&
        "message" in err &&
        typeof err.message === "string") {
        return err.message;
    }
    if (typeof err === "object" &&
        "stack" in err &&
        typeof err.stack === "string") {
        return err.stack.split("\n")[0];
    }
    return String(err);
}
exports.getErrorMessage = getErrorMessage;
//# sourceMappingURL=indexedDB.js.map